/*******************************************************************************
*
* File Name:   sensoren.c
* Project  :   c't Bot library
*
* Ver.     Date                 Author                    Comments
* -------  ----------   ---------------------   ------------------------------
* 2.00	   28.06.2006   Yang Li & Xingxing Hu   build
*                                               Initialisiere alle Sensore -> alle_sensoren_init(void);
*                                               geschwindigkeit-funktion -> speed_berechnung(void);
*												positionsfunktion -> position_bestimmung(void);
*                                               berechnung der Entfernung -> speed_berechnung(void);
*                                               sensoren_isr(void) -> encoder_isr(void);
*                                               wert fuer encoder stetzen -> encoder_Set(int setl,int setr);
*
*
* 2006 Institut fuer Informatik, Tu-Clausthal
*****************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 ***************************************************************************/

#include "sensoren.h"

/* Initialisiere alle Sensore */
void alle_sensoren_init(void){

    ena_init();
	adc_init();		// Alle ADC-Ports aktivieren


	ena_set(ENA_RADLED);		// Alle Sensoren bis auf die Radencoder deaktivieren
	ena_ein(ENA_ABSTAND);		// Die Abstandssensoren ebenfalls dauerhaft an, da sie fast 50 ms zum booten brauchen


	// ---------- maussensor init ------------------------------------------
	maus_init();


	// ---------- analog(ueber adc) sensoren init ---------------------------
	DDRA = 0x00;     //PortA als eingang
    PORTA = 0x00;    //portA0 bis portA7 ausschalten


	// ---------- radsensor init -------------------------------------------
	DDRB &= ~ (1<<4);	// Input(DDR fuer den linken Encoder)
	DDRD &= ~(1<<3);	// Input(DDR fuer den rechten Encoder)


	// ---------- klappesensor init ----------------------------------------
	DDRD &= 0xbf;


	// ---------- schrank(Transportfachueberwachung) init ------------------
    DDRB &= 0xfe;
	PORTB |= (1<<0);

	// ---------- sensor zur Ueberwachung Motor oder Batteriefehler init ---
	DDRB &= ~(1<<2);

	timer2_init();

}


/* geschwindigkeit-funktion*/
void speed_berechnung(void){

   static int lastTime =0,encoder_l_alt =0,encoder_r_alt =0;


	if (zeit_s() != lastTime) {	// sollte genau 1x pro Sekunde zutreffen

		Speed_l=  ((sensEnc_data[0] - encoder_l_alt) * 179) / 60; //wieso neue encoderstuecke - alte encoderstuecke * 3mm ?
		Speed_r= ((sensEnc_data[1] - encoder_r_alt) * 179) / 60;

		encoder_l_alt= sensEnc_data[0];    // jetzige Encoder-wert wird als alte Encoder-Wert für nächste Periode festgelegt
		encoder_r_alt= sensEnc_data[1];




		lastTime = zeit_s();
	}
}


/* positionsfunktion*/
void position_bestimmung(void){

    Maus_data[0] = maus_read(MOUSE_DELTA_X_REG);
	Maus_data[1] = maus_read(MOUSE_DELTA_Y_REG);
	if (Maus_data[0]<=127){
	    Maus_X += Maus_data[0];
	}
	if (Maus_data[0]>127){
	    Maus_X -= Maus_data[0];
	}
	if (Maus_data[1]<=127){
	    Maus_Y += Maus_data[1];
	}
	if (Maus_data[1]>127){
	    Maus_Y -= Maus_data[1];
	}
}


/* berechnung der Entfernung:D = a/(x-b)
 * D ist die Entfernung,X ist der Ausgabewert des Sensors
 * A ist die Steigung der Kurve A/X
 * B ist der Offset der Kurve
 */
int entfernung(int ausgabewert){

    #define a	  5149		//Wert bekommt man durch experiment a=(x1-x2)*D2*D1/(D1-D2)
    #define b	  17.554	    //Wert bekommt man durch experiment b=(D2*x2-D1*x1)/(D2-D1)

	return a / (ausgabewert - b);
}


/* Interruptroutine fuer Sensoren (ohne Radsensor)*/
void sensoren_isr(void){

	ena_ein(ENA_KANTLED|ENA_MAUS|ENA_SCHRANKE|ENA_KLAPPLED);
	ADCSRA |= (1 << ADSC);            //    A/D wandler einschalten

	//--------------------- klappesensor und transportfachsensor -----------------------
	Klappesensor_data[0] = ((PIND >> 6) & 0x01);
	Transportfachsensor_data[0] = ((PINB >> 0) & 0x01) & 0x01;

	//--------------------- fehlerueberwachung -----------------
	SensError_data[0] = (PINB >> 2) & 0x01;

}


/* Interruptroutine fuer radsensor */
void encoder_isr(void)
{    unsigned int enc_l=0,enc_r=0;
     unsigned int neu_PINB, neu_PIND;

	// --------------------- links ----------------------------
	//Rad-Encoder auswerten
    neu_PINB = PINB >>4;
	if ( (neu_PINB & 0x01) != enc_l)				// uns interessieren nur Veraenderungen
	{
		enc_l=(neu_PINB & 0x01);					// neuen Wert sichern

		if (sl==1)	             // linke Geschwindigkeit positiv
			sensEnc_data[0]++;	// vorwaerts zählen
	    if (sl==0)	            // linke Geschwindigkeit negativ
			sensEnc_data[0]--;	// rueckwaerts zählen
		if (sl==2)              // linke Geschwindigkeit = 0
			sensEnc_data[0]=sensEnc_data[0];	// Encoderdaten werden nicht geändert
	}


	// --------------------- rechts ----------------------------
	neu_PIND = PIND >>4;
	if ((neu_PIND & 0x01) != enc_r)					// uns interessieren nur Veraenderungen
	{
		enc_r=(neu_PIND & 0x01);					// neuen wert sichern

		if (sr==0)	// Drehrichtung beachten
			sensEnc_data[1]++;	//vorwaerts
	    if (sr==1)
			sensEnc_data[1]--;	//rueckwaerts
		if (sr==2)
			sensEnc_data[1]=sensEnc_data[1];	//stop
	}
	speed_berechnung();
    }


void encoder_Set(int setl,int setr)
{
	sensEnc_data[0]=setl;
	sensEnc_data[1]=setr;
}


