/*******************************************************************************
*
* File Name:   timer.c
* Project  :   c't Bot library
*
*
* Ver.     Date                 Author                    Comments
* -------  ----------   ---------------------   ------------------------------
* 2.00	   28.06.2006   Yang Li & Xingxing Hu   build
*                                               Diese Funktion liefert den Mikrosekundenanteil der Systemzeit zurueck -> zeit_us(void);
*                                               Diese Funktion liefert den Millisekundenanteil der Systemzeit zurueck -> zeit_ms(void);
*                                               Diese Funktion liefert den Sekundenanteil der Systemzeit zurueck -> zeit_s(void);
*                                               Funktion, um die Systemzeit zu berechnen -> systemzeit_isr(void);
*                                               Initialisiert Timer 2 und startet ihn -> timer2_init(void);
*
* 2006 Institut fuer Informatik, Tu-Clausthal
*****************************************************************************/
/***************************************************************************
*
*the timer source is based on timer.c ver 1.0 by Benjamin Benz 20.12.05
*Copyright (c) 2005, 59 Temple Place, Suite 330, Boston,MA 02111-1307, USA.
*
****************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 ***************************************************************************/

#include "timer.h"

static volatile unsigned int zeit_in_us=0;   /* Mikrosekundenanteil an der Systemzeit */
static volatile unsigned int zeit_in_ms=0;   /* Millisekundenanteil an der Systemzeit */
static volatile unsigned int zeit_in_s=0;    /* Sekundenanteil an der Systemzeit */



/* initilaisiert Timer 2 und startet ihn */
void timer2_init(void)
{
	// aendert man den Prescaler muss man die Formel fuer OCR2 anpassen !!!
	// use CLK/64 prescale value, clear timer/counter on compare match
	TCCR2 = (1 << WGM21) | (1 << CS22);
	OCR2 = ((F_CPU/64/TIMER2_CLOCK) - 1 );
	TIMSK  |= (1 << OCIE2);							// enable Output Compare 0 overflow interrupt
}

/* initilaisiert Timer 0 und startet ihn */
void timer0_init(void)
{
    // use CLK/64 prescale value, clear timer/counter on compare match
	TCCR0 = (1 << WGM01) | (1 << CS01) | (1 << CS00);
	OCR0 = ((F_CPU/64/TIMER0_CLOCK) - 1 );
	TIMSK  |= (1 << OCIE0);							// enable Output Compare 0 overflow interrupt
}



/* Diese Funktion liefert den Mikrosekundenanteil der Systemzeit zurueck.*/
inline unsigned int zeit_us(void)
{
   unsigned int temp;
   LOCK();
   temp = zeit_in_us;
   UNLOCK();
   return temp;
}

/* Diese Funktion liefert den Millisekundenanteil der Systemzeit zurueck.*/
inline unsigned int zeit_ms(void)
{
	unsigned int temp;
	LOCK();
	temp = zeit_in_ms;
	UNLOCK();
	return temp;
}

/* Diese Funktion liefert den Sekundenanteil der Systemzeit zurueck.*/
inline unsigned int zeit_s(void)
{
	unsigned int temp;
	LOCK();
	temp = zeit_in_s;
	UNLOCK();
	return temp;
}

/* Funktion, um die Systemzeit zu berechnen */
void systemzeit_isr(void)
{
   	LOCK();
   	zeit_in_us += TIMER_STEPS;		/* Mikrosekundentimer erhoehen */
   	if (zeit_in_us >= 1000)      		/* Eine Mikrosekunde verstrichen? */
	{
    	zeit_in_ms += (zeit_in_us/1000);
      	zeit_in_us %= 1000;      		/* alle vollen Millisekunden vom Mikrosekundenzaehler in den Millisekudnenzaehler verschieben */
      	if (zeit_in_ms == 1000)   			/* Eine Sekunde verstrichen? */
		{
        	zeit_in_ms=0;
         	zeit_in_s++;
      	}
   	}
   	UNLOCK();
}



