/*******************************************************************************
*
* File Name:   timer.h
* Project  :   c't Bot library
*
*
* Ver.     Date                 Author                    Comments
* -------  ----------   ---------------------   ------------------------------
* 2.00	   28.06.2006   Yang Li & Xingxing Hu   build
*                                               Diese Funktion liefert den Mikrosekundenanteil der Systemzeit zurueck -> zeit_us(void);
*                                               Diese Funktion liefert den Millisekundenanteil der Systemzeit zurueck -> zeit_ms(void);
*                                               Diese Funktion liefert den Sekundenanteil der Systemzeit zurueck -> zeit_s(void);
*                                               Funktion, um die Systemzeit zu berechnen -> systemzeit_isr(void);
*                                               Initialisiert Timer 2 und startet ihn -> timer2_init(void);
*
*
* 2006 Institut fuer Informatik, Tu-Clausthal
*****************************************************************************/
/***************************************************************************
*
*the timer source is based on timer.c ver 1.0 by Benjamin Benz 20.12.05
*Copyright (c) 2005, 59 Temple Place, Suite 330, Boston,MA 02111-1307, USA.
*
****************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 ***************************************************************************/

#ifndef TIMER_H_
#define TIMER_H_

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include "sensoren.h"

#define LOCK()      		/* ISR :Schuetzt die Zeitsynchronisation */
#define UNLOCK()   			/* ISR :Hebt den Schutz fuer die Zeitsynchronisation wieder auf */
#define F_CPU	16000000UL    /* Crystal frequency in Hz */

/* Diese Funktion liefert den Mikrosekundenanteil der Systemzeit zurueck.*/
inline unsigned int zeit_us(void);

/* Diese Funktion liefert den Millisekundenanteil der Systemzeit zurueck.*/
inline unsigned int zeit_ms(void);

/* Diese Funktion liefert den Sekundenanteil der Systemzeit zurueck.*/
inline unsigned int zeit_s(void);

/* Funktion, um die Systemzeit zu berechnen */
void systemzeit_isr(void);

// Die Werte fuer TIMER_X_CLOCK sind Angaben in Hz

/* Frequenz von Timer 2 in Hz */
#define TIMER2_CLOCK	5586	// alle 179 �s einen Interrupt zur Abfrage der Rad-Encoder
#define TIMER0_CLOCK	5000	// alle 200 �s einen Interrupt zur Abfrage der maus
/* Mikrosekunden die zwischen zwei Timer-Aufrufen liegen */
#define TIMER_STEPS	(1000000/TIMER2_CLOCK)

/* Initialisiert Timer 2 und startet ihn */
void timer2_init(void);
void timer0_init(void);

#endif
