/*******************************************************************************
*
* File Name:   uart.c
* Project  :   c't Bot library
*
*
* Ver.     Date                 Author                    Comments
* -------  ----------   ---------------------   ------------------------------
* 2.00	   28.06.2006   Yang Li & Xingxing Hu   build
*                                               uart_Initialisieren -> uart_init();
*                                               daten senden -> CTWrite(unsigned char *data,unsigned char length);
*                                               daten empfangen -> CTRead(unsigned char *data, unsigned char length);
*                                               print -> PrintInt(int wert);
*
* Copyright (c) 2006 Institut f�r Informatik, Tu-Clausthal
*****************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 ***************************************************************************/

#include "uart.h"
#include <string.h>

/* USART initialisieren */
void uart_init(void)
{
    PORTD|= 0x03;
	DDRD |= 0x02;

	/* Aktivieren von receiver und transmitter */
	UCSRB = (1<<RXEN)|(1<<TXEN)|(1 << RXCIE);

	/* Einstellen des Datenformats: 8 Datenbits, 1 Stoppbit */
	UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);

	/* Baudrate einstellen ( Normaler Modus ) */
	UBRRH = (unsigned char) (UBRR_BAUD>>8);
	UBRRL = (unsigned char) UBRR_BAUD;
}

/* Funktion fuer serielle Kommunikation */
void CTWrite(unsigned char *data, unsigned char length)          //UART
{
    unsigned char i = 0;
	while (length > 0)
   {

	while ( !( UCSRA & (1<<UDRE)) );       /* Warten bis der Sendepuffer frei ist */

	UDR = data[i++];       /* Daten in den Puffer schreiben und damit senden */

	length --;

	}
	/*while (!(UCSRA & 0x40));
	for (i = 0; i < 0xFE; i++)
		for(length = 0; length < 0xFE; length++); */
}

void CTRead(unsigned char *data, unsigned char length)
{
	unsigned char i = 0;

		while (i < length) {
			while ( !(UCSRA & (1<<RXC)) );
				data[i++] = UDR;
		}
}

void PrintInt(long int wert)
{  	char text[10]="         ";
	ltoa(wert,text,10);      //Diese Funktionen wandelt eine Zahl vom angegebenen Format in einen String.
	CTWrite(text,strlen(text)); // vorher wurde 0 gesendet aus irgendwelchen Gründen Verwirrung für gtkTerm, deshalb Hochzählen des textes bis die 0 kommmt
}

// Zeichen senden
int uart_putc(unsigned char c)
{
    while (!(UCSRA & (1<<UDRE))); /* warten bis Senden moeglich */
    UDR = c;                      /* sende Zeichen */
    return 0;
}

// String senden
void uart_puts (char *s)
{
    while (*s)
    {   /* so lange *s != '\0' also ungleich dem "String-Endezeichen" */
        uart_putc(*s);
        s++;
    }
}

//Zeichen empfangen
uint8_t uart_getc (void)
{
    while (!(UCSRA & (1<<RXC)));  // warten bis Zeichen verfuegbar
    return UDR;                   // Zeichen aus UDR an Aufrufer zurueckgeben
}



